--[[
Pet.lua
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2009 by Horacio Hoyos

This file is part of DHUD3.

    DHUD3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD3.  If not, see <http://www.gnu.org/licenses/>.
]]
local DHUD3 = LibStub("AceAddon-3.0"):GetAddon("DHUD3");
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD3");
local DogTag = LibStub("LibDogTag-3.0");

local MODNAME = "Pet";
local Pet = DHUD3:NewModule(MODNAME, "AceEvent-3.0");

local string_match = string.match;
local string_find = string.find;
local string_len = string.len;
local string_sub = string.sub;
local defaultfont = "Fonts/FRIZQT__.TTF"
local layout = {
        health = "e",
        power = "e",
        icons = "e",
    }

	--Local Defaults
local db
local defaults = {
	profile = {
        blizPet = false,
        pet = "c",
		icon = true,
		layout = "HLPR",
		barHText = true,
		barPText = true,
		hStyle = "[FractionalHP] ([PercentHP:Percent])",
		cHStyle = "",
		pStyle = "[FractionalMP] ([PercentMP:Percent])",
		cPStyle = "",
		dStyle = "[FractionalMP] ([PercentMP:Percent])",
		dCStyle = "",
		vehicle = true,
		vehBars =  "player",
		druidMana = true,
		druidBar = "r",
		petBarsText = {
			lpo = {},
			lpi = {},
			rpo = {},
			rpi = {},
		},
		textSize = 10,
		colors = {
			["0"] = {
	            full = { 0, 0.67, 0.67 },
	            med = { 0, 0, 0.67 },
	            low = { 0.67, 0, 0.67 }
	        },
			["1"] = { 
	            full = { 0.67, 0, 0 },
	            med = { 0.67, 0.15, 0.15 },
	            low = { 0.67, 0.28, 0.28 }
	        },
			["2"] = {
	            full = { 0.67, 0.27, 0 },
	            med = { 0.67, 0.27, 0 },
	            low = { 0.67, 0.27, 0 }
	        },
	        ["3"] = { 
	            full = { 0.67, 0.67, 0 },
	            med = { 0.67, 0.67, 0.15 },
	            low = { 0.67, 0.67, 0.28 }
	        },
			["6"] = { 
	            full = { 0, 0.43, 0.58 },
	            med = { 0.15, 0.48, 0.56 },
	            low = { 0.21, 0.50, 0.54 }
	        },
	        ["7"] = {
	            full = { 0, 0.67, 0 },
	            med = { 0.67, 0.67, 0 },
	            low = { 0.67, 0, 0 },
	        },
		},
	}
}

-- Functions required by the configuration options table
local OptGetter, OptSetter, ColorGetter, ColorSetter, SelectGetter, SelectSetter, layoutType, healthTags, powerTags;
do
	local mod = Player
	function OptGetter(info)
		local key = info[#info];
		return db[key];
	end

	function OptSetter(info, value)
		local key = info[#info];
		db[key] = value;
		mod:Refresh();
	end
    
    function ColorGetter (info)
        local key1, key2, key3 = info[#info-2], info[#info-1], info[#info]
		return db[key1][key2][key3][1], db[key1][key2][key3][2], db[key1][key2][key3][3]
	end
    
    function ColorSetter (info, r, g, b) 
        local key1, key2, key3 = info[#info-2], info[#info-1], info[#info]
        db[key1][key2][key3][1] = r
        db[key1][key2][key3][2] = g
        db[key1][key2][key3][3] = b
        mod:Refresh();
    end
	
	layoutType = {["c"] = L["Centered"], ["r"] = L["Right"], ["l"] = L["Left"]}
    healthTags = {
		["[FractionalHP] ([PercentHP:Percent])"] = "Current/Max (Percent%)",
		["[FractionalHP]"] = "Current/Max",
		["[HP] ([PercentHP:Percent])"] = "Current (Percent%)",
		["[PercentHP:Percent]"] = "Percent%",
		["[HP]"] = "Current"
	}
	powerTags = {
		["[FractionalMP] ([PercentMP:Percent])"] = "Current/Max (Percent%)",
		["[FractionalMP]"] = "Current/Max",
		["[MP] ([PercentMP:Percent])"] = "Current (Percent%)",
		["[PercentMP:Percent]"] = "Percent%", 
		["[MP]"] = "Current"
	}
	vehicleDisplay = {["player"] = L["Player"], ["vehicle"] = L["Vehicle"]}
end


--Local Options
local options
local function GetOptions()
	if not options then
        --Options table
    	options = {
    		type = "group",
			name = L["The pet module manages the pet bars functionality. Track pet, vehicle or druid mana"],
			arg = MODNAME,
            order = 2,
			get = OptGetter,
			set = OptSetter,
    		args = {
                enabled = {
                    order = 1,
                    type = "toggle",
                    name = L["Enable Pet Module"],
                    get = function() return DHUD3:GetModuleEnabled(MODNAME) end,
                    set = function(info, value) DHUD3:SetModuleEnabled(MODNAME, value) end,
                },
				general = {
					type = 'group',
					order = 1,
					name = L["Layout"],
					args = {
						pet = {
							type = 'select',
							order = 1,
							name = L["PetBars"],
							desc = L["Place the Pet's bars."],
							set = function (info, value) 
									db[info[#info]] = value
									if string_match(value, "c") then
										db.layout = "HLPR";
									else
										db.layout = "HIPM";
									end
									Pet:Refresh();
								end,
							values = layoutType,
						},
						layout = {
							type = 'select',
							order = 2,
							name = L["Bar Layout"],
							values = function ()
									if string_match(db.pet, "c") then
										return { ["HLPR"] = L["Health Left/Power Right"], ["HRPL"] = L["Health Right/Power Left"] }
									else
										return { ["HIPO"] = L["Health Inner/Power Outter"], ["HOPI"] = L["Health Outter/Power Inner"] }
									end
								end,
						},
						icon = {
							type = 'toggle',
							order = 3,
							name = L["Pet Happiness Icon"],
							desc = L["Show/Hide"],
						},
					},
				},
				barText = {
					type = 'group',
					order = 2,
					name = L["Bar's Text"],
					desc = L["Configure Health and Power (mana/rage/energy/runic) text options"],
					args = {
						pBarText = {
							type = 'multiselect',
							order = 1,
							name = L["Show Bar Values"],
							values = { ["pBarHText"] = L["Health"], ["pBarPText"] = L["Power"] },
							get = SelectGetter,
							set = SelectSetter,
						},
						pHStyle = {
							type = 'select',
							order = 2,
							name = L["Health Style"],
							hidden = BarTextHidden,
							values = healthTags
							},
						pCHStyle = {
							type = 'input',
							order = 3,
							name = L["Custom Health Style"],
							desc = L["If set this style will override Health Style value. Use LibDogTag-3.o Tags. (Note: Leave/set blank to use Heatlh Style setting.)"],
							hidden = BarTextHidden,
							usage = L["Enter /dog for in-game LibDogTag-3.0 Tag documentation."],							
						},
						pPStyle = {
							type = 'select',
							order = 4,
							name = L["Power Style"],
							hidden = BarTextHidden,
							values = powerTags
							},
						pCPStyle = {
							type = 'input',
							order = 5,
							name = L["Custom Power Style"],
							desc = L["If set this style will override Power Style value. Use LibDogTag-3.o Tags. (Note: Leave/set blank to use Heatlh Style setting.)"],
							hidden = BarTextHidden,
							usage = L["Enter /dog for in-game LibDogTag-3.0 Tag documentation."],								
						},
						pBarTextSize = {
							type = 'range',
							order = 6,
							name = L["Font size"],
							hidden = BarTextHidden,
							min = minfont,
							max = maxfont,
							step = 1,
						},
					},
				},	
				colors = {
					type = 'group',
					order = 3,
					name = L["Colors"],
					get = ColorGetter,
					set = ColorSetter,
					args = {
						["7"] = {
							type = 'group',
							name = L["Health"],
							order = 1,
							guiInline  = true,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
						["0"] = {
							type = 'group',
							name = L["Mana"],
							guiInline  = true,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
						["1"] = {
							type = 'group',
							name = L["Rage"],
							guiInline  = true,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
						["3"] = { 
							type = 'group',
							name = L["Energy"],
							guiInline  = true,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
						["6"] = { 
							type = 'group',
							name = L["Runic Power"],
							guiInline  = true,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
					},
				},
				vehicle = {
					type = 'group',
					order = 4,
					name = L["Vechile Tracking"],
					desc = L["Configure Vehicle/Player options when in vehicle"],
					args = {
						vehicle = {
							type = 'toggle',
							order = 1,
							name = L["In Vechile"],
							desc = L["Use pet bars to track vehcile/player stats"],
						},
						vehBars = {
							type = 'select',
							order = 1,
							name = L["Unit"],
							desc = L["Unit Pet bars track."],
							values = vehicleDisplay,
						},
					},
				},
				druid = {
					type = 'group',
					order = 5,
					name = L["Vechile Tracking"],
					desc = L["Configure Vehicle/Player options when in vehicle"],
					args = {
						druidMana = {
							type = 'toggle',
							name = L["Druid Mana"],
							desc = L["Track druid's mana when shape shifted"],
							order = 1,
						},
						druidBar = {
							type = 'select',
							order = 2,
							name = L["Side"],
							desc = L["Bar Side to track mana"],
							values = {["r"] = L["Right"], ["l"] = L["Left"]},
						},
					},
				},
			},
		}
	end
	return options;
end

-- Local functions
local PlaceBarTexts, ConfigBars;
do
	-- Place the bar's text if player moved them
    function PlaceBarTexts()
    	
		local barText = DHUD3.layout.petText;
		local position
		for k, v in pairs(barText) do
			position = DHUD3.db.profile.barTextPos[k];
			if position.moved then
				ref = _G["DHUD3_"..k.."_Text"];
				point, parent, relative = unpack(DHUD3.layout.petText);
	    		ref:ClearAllPoints();
	    		ref:SetPoint(point, _G["DHUD3_"..parent], relative, position.x, position.y);
	    	end
    	end
    end
	
	-- Asing each bar the their respective tracking stat accordingly to the saved profile
    function ConfigBars()
	
        --DHUD3:Debug("ConfigBars");
        -- Pet
    	if string_match(db.pet, "c") then
    		if string_match(db.layout, "HLPR") then
                layout.health = "lpo";
                layout.power = "rpo";
                layout.icons = "l";
    		else
                layout.health = "rpo";
                layout.power = "lpo";
                layout.icons = "r";
    		end
    	elseif string_match(db.player, "l") then
            layout.icons = "l";
            if string_match(db.pLayout, "HIPO") then
                layout.health = "lpi";
                layout.power = "lpo";
    		else
                layout.health = "lpo";
                layout.power = "lpi";
            end
    	else
    		layout.icons = "r";
            if string_match(db.pLayout, "HIPO") then
                layout.health = "rpi";
                layout.power = "rpo";
    		else
                layout.health = "rpo";
                layout.power = "rpi";
            end
        end
    	
    	--Bars Text Tag
    	if db.barHText then
            _G["DHUD3_"..layout.health.."_Text"]:Show();
    		local tagCode = (string_len(db.cHStyle) > 0) and db.cHStyle or db.hStyle;
    		DogTag:AddFontString(_G["DHUD3_"..layout.health.."_Text_Text"], _G["DHUD3_"..layout.health.."_Text"], tagCode, "Unit", { unit = "pet" } );
        else
            DogTag:RemoveFontString(_G["DHUD3_"..layout.health.."_Text_Text"]);
            _G["DHUD3_"..layout.health.."_Text"]:Hide();
    	end
    	if db.barPText then
            _G["DHUD3_"..layout.power.."_Text"]:Show();
    		local tagCode = (string_len(db.cPStyle) > 0) and db.cPStyle or db.pStyle;
    		DogTag:AddFontString(_G["DHUD3_"..layout.power.."_Text_Text"], _G["DHUD3_"..layout.power.."_Text"], tagCode, "Unit", { unit = "pet" });
        else
            DogTag:RemoveFontString(_G["DHUD3_"..layout.power.."_Text_Text"]);
            _G["DHUD3_"..layout.power.."_Text"]:Hide();
    	end
		if db.druidMana then
			_G["DHUD3_"..db.druidBar.."pod_Text"]:Show();
			_G["DHUD3_"..db.druidBar.."pod_Text"]:SetAlpha(0);
        else
            _G["DHUD3_"..db.druidBar.."pod_Text"]:Hide();
		end
    end
	
	function UpdatePetIcon()
	
		local happ = GetPetHappiness();
        local health = UnitHealth("pet");
	    --DHUD3:Debug("UpdatePetIcon", health, happ);
	    if health > 0 and happiness then
	        local texture, x0, x1, y0, y1 = unpack( DHUD3.layout.petTextures[happ] );
	        if ( texture ) then
				_G["DHUD3_"..layout.icons.."pet_Texture"]:SetTexture(texture);
	            _G["DHUD3_"..layout.icons.."pet_Texture"]:SetTexCoord(x0, x1, y0, y1);
	            _G["DHUD3_"..layout.icons.."pet"]:Show();
	        else
	            _G["DHUD3_"..layout.icons.."pet"]:Hide();
	        end
	    else
	        _G["DHUD3_"..layout.icons.."pet"]:Hide();
	    end
	end
	
	function SetBars(unit, healthBar, powerBar)

		local maxVal, curFrac,needH, needP;
		local color = {r=0,g=0,b=0,hex=""};
		-- Health
	    maxVal, curFrac, needH = DHUD3:GetUnitHealth(unit);
	    local colors = db.colors[tostring(7)];
	    color.r, color.g, color.b, color.hex = DHUD3:Colorize(colors, curFrac);
	    DHUD3:UpdateBar(healthBar, maxVal, curFrac, color);
	    local bar = _G["DHUD3_"..healthBar]
	    bar:UnregisterAllEvents();
	    bar.who = unit;
	    bar.bar = healthBar;
	    bar.colors = colors;
	    bar:RegisterEvent("UNIT_HEALTH");
	    bar:RegisterEvent("UNIT_MAXHEALTH");
	    bar:SetScript("OnEvent", DHUD3.OnUnitHealth);
	    bar:Show();
	    --Power
	    maxVal, curFrac, needP = DHUD3:GetUnitPower(unit);
		local powerType = UnitPowerType(unit);
	    colors = db.colors;
	    color.r, color.g, color.b, color.hex = DHUD3:Colorize(colors[tostring(powerType)], curFrac);
	    DHUD3:UpdateBar(powerBar, maxVal, curFrac, color);    
	    bar = _G["DHUD3_"..powerBar]
	    bar:UnregisterAllEvents();
	    bar.who = unit;
	    bar.bar = layout.power;
	    bar.maxVal = maxVal;
	    bar.curFrac = curFrac;
	    bar.display = powerType;
	    bar.colors = colors;
		if ( maxVal ~= 0 ) then 
			bar:SetScript("OnUpdate", DHUD3.OnUpdateBarPower);
			local powerType = UnitPowerType(unit);
			bar.display = powerType;
			bar.colors = colors;
			if ( powerType == 0 ) then
				bar:RegisterEvent("UNIT_MAXMANA");
			elseif ( powerType == 1 ) then
				bar:RegisterEvent("UNIT_MAXRAGE");
			elseif ( powerType == 3 ) then
				bar:RegisterEvent("UNIT_MAXENERGY");
			elseif ( powerType == 6 ) then
				bar:RegisterEvent("UNIT_MAXRUNIC_POWER");
			end
			bar:RegisterEvent("UNIT_DISPLAYPOWER");
			bar:SetScript("OnEvent", DHUD3.OnUnitPower);
		else
			bar:SetScript("OnUpdate", nil);
		end
		bar:Show();
	end
end

function Pet:OnInitialize()
	self.db = DHUD3.db:RegisterNamespace(MODNAME, defaults);
	db = self.db.profile;
	
	self:SetEnabledState(DHUD3:GetModuleEnabled(MODNAME));
	DHUD3:RegisterModuleOptions(MODNAME, GetOptions, L["Player"]);
	
    local bars = DHUD3.layout.petBars;
    local barText = DHUD3.layout.petText;
    local point, parent, relative, x, y, width, height;
	local icons = DHUD3.layout.petIcon;
	
	-- Bars
    for k, v in pairs(bars) do
        point, parent, relative, position, texture, border = unpack(v);
        x0, x1, y0, y1 = unpack(DHUD3.layout[position]);
        ref = CreateFrame("Button", "DHUD3_"..k, _G["DHUD3_"..parent]);
        ref:ClearAllPoints();
        ref:SetPoint(point, _G["DHUD3_"..parent], relative, 0, 0);
        ref:SetHeight(256);
        ref:SetWidth(128);
        bgt = ref:CreateTexture("DHUD3_"..k.."_Texture", "ARTWORK");
        bgt:SetTexture(texture);
        bgt:SetPoint(point, ref, relative, 0, 0);
        bgt:SetHeight(256);
        bgt:SetWidth(128);
        bgt:SetTexCoord(x0, x1, y0, y1);
        bgt:SetBlendMode("BLEND");
        bgt = ref:CreateTexture("DHUD3_"..k.."_Border", "BACKGROUND");
        bgt:SetTexture(border);
        bgt:SetPoint(point, ref, relative, 0, 0);
        bgt:SetHeight(256);
        bgt:SetWidth(128);
        bgt:SetTexCoord(x0, x1, y0, y1);
        bgt:SetBlendMode("BLEND");
        ref:EnableMouse(false);
        ref:Hide();
		ref:SetScript("OnShow", OnBarShow);
		ref:SetScript("OnHide", OnBarHide);
    end
	
	-- Bar's Text
    for k, v in pairs(barText) do
        point, parent, relative, x, y, width, height, justify = unpack(v);
        ref = CreateFrame("Frame", "DHUD3_"..k.."_Text", _G["DHUD3_"..parent]);
        ref:SetWidth(width);
        ref:SetHeight(height);
        ref:ClearAllPoints();
        ref:SetPoint(point, _G["DHUD3_"..parent], relative, x, y);
        local font = ref:CreateFontString("DHUD3_"..k.."_Text_Text", "OVERLAY");
        font:SetFontObject(GameFontHighlightSmall);
        font:ClearAllPoints();
        font:SetPoint(justify, ref, justify, 0, 0);
		font:SetText(k.."2.5 Lets try the aligment");
        font:SetWidth(font:GetStringWidth() * 1.1);
        font:SetJustifyH(justify);
        font:Show();
        ref.isMoving = false;
		ref.dropTimer = 0;
		ref:RegisterForDrag("LeftButton");
		ref:SetMovable(true);
		ref:SetScript("OnDragStart", DHUD3.MoveFrame);
		ref:SetScript("OnDragStop", DHUD3.DropFrame);
        ref:EnableMouse(true);
        ref:Hide();
    end
	
	-- Icons
    for k, v in pairs(icons) do
        parent, x, y = unpack(v);
        ref = CreateFrame("Button", "DHUD3_"..k, _G["DHUD3_"..parent]);
        ref:ClearAllPoints();
        ref:SetPoint("TOP", _G["DHUD3_"..parent], "TOP", x, y);
        ref:SetHeight(20);
        ref:SetWidth(20);
		local bgt = ref:CreateTexture("DHUD3_"..k.."_Texture","BACKGROUND")
        bgt:ClearAllPoints()
        bgt:SetPoint("TOPLEFT", ref , "TOPLEFT", 0, 0)
        bgt:SetPoint("BOTTOMRIGHT", ref , "BOTTOMRIGHT", 0, 0)
        ref:Hide();
    end
end

function Pet:OnEnable()
	
	--DHUD3:Debug("Pet:OnEnable");	
    PlaceBarTexts();
    self:Refresh();
	if not self:IsEnabled() then return end
	
end

function Pet:OnDisable()
	
	self:UnregisterAllEvents();
    DHUD3_lpi:Hide();
    DHUD3_lpo:Hide();
    DHUD3_rpi:Hide();
    DHUD3_rpo:Hide();
    DHUD3_lpi:SetScript("OnUpdate", nil);
    DHUD3_lpo:SetScript("OnUpdate", nil);
    DHUD3_rpi:SetScript("OnUpdate", nil);
    DHUD3_rpo:SetScript("OnUpdate", nil);
end

function Pet:Refresh()

	db = self.db.profile
	if not self:IsEnabled() then return end
    
    ConfigBars();
    DHUD3_lpi:Hide();
    DHUD3_lpo:Hide();
    DHUD3_rpi:Hide();
    DHUD3_rpo:Hide();
    DHUD3_lpi:SetScript("OnUpdate", nil);
    DHUD3_lpo:SetScript("OnUpdate", nil);
    DHUD3_rpi:SetScript("OnUpdate", nil);
    DHUD3_rpo:SetScript("OnUpdate", nil);
    
    -- Fonts
    local font
    local w
    --Targer Text
    local scale = DHUD3.GetScale();
    -- Pet Bar's Text
    font = _G["DHUD3_"..layout.health.."_Text_Text"]
    font:SetFont(defaultfont, db.textSize * scale)
    font:SetWidth(200)
    font = _G["DHUD3_"..layout.power.."_Text_Text"]
    font:SetFont(defaultfont, db.textSize * scale)
    font:SetWidth(200)
    
    -- Pet
	    -- Pet Status
    DHUD3_lpet:Hide();
    DHUD3_rpet:Hide();
	
	if ( UnitExists("pet") ) then
		SetBars("pet", layout.health, layout.power);
		UpdatePetIcon();
    end
	self:RegisterEvent("UNIT_PET");
	
    -- Vehicle Swap
    if db.vehicle then
        self:RegisterEvent("UNIT_ENTERED_VEHICLE");
        self:RegisterEvent("UNIT_EXITED_VEHICLE");
        if ( UnitInVehicle("player") ) then
            SetBars(db.vehBars);
        end
    else
        self:UnregisterEvent("UNIT_ENTERED_VEHICLE");
        self:UnregisterEvent("UNIT_EXITED_VEHICLE");
    end
	
	--Druid
	local _, class = UnitClass("player");
	if ( db.druidMana and ( class ~= "DRUID" ) ) then
		self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
		DHUD3:Debug("Druid mana bar");
		-- Mostrarlas de una?
		local powerType = UnitPowerType("player");
		if ( powerType ~= 0 ) then
			self:UPDATE_SHAPESHIFT_FORM();
		end
	end
end

function Pet:UNIT_PET(event, who)
	
	if ( ( who == "player" ) and UnitExists("pet") ) then
		SetBars("pet", layout.health, layout.power);
		UpdatePetIcon();
	else
		_G["DHUD3_"..layout.health]:UnregisterAllEvents();
		_G["DHUD3_"..layout.health]:SetScript("OnUpdate", nil);
		_G["DHUD3_"..layout.health]:Hide();
		_G["DHUD3_"..layout.power]:UnregisterAllEvents();
		_G["DHUD3_"..layout.power]:SetScript("OnUpdate", nil);
		_G["DHUD3_"..layout.power]:Hide();
    end	
end

function Pet:UNIT_HAPPINESS(event, who)
	
    if ( who == "pet" ) then
        UpdatePetIcon();
    end
end

function Pet:UNIT_ENTERED_VEHICLE(event, who)
	
	--DHUD3:Debug("UNIT_ENTERED_VEHICLE");
    if who == "player" then
        if ( UnitInVehicle("player") ) then
            SetBars(db.vehBars, layout.health, layout.power);
        end
    end
end

function Pet:UNIT_EXITED_VEHICLE(event, who)
	
	--DHUD3:Debug("UNIT_EXITED_VEHICLE");
    if ( who == "player" ) then
        if ( UnitExists("pet") ) then
			SetBars("pet", layout.health, layout.power);
		else
			_G["DHUD3_"..layout.health]:UnregisterAllEvents();
			_G["DHUD3_"..layout.health]:SetScript("OnUpdate", nil);
			_G["DHUD3_"..layout.health]:Hide();
			_G["DHUD3_"..layout.power]:UnregisterAllEvents();
			_G["DHUD3_"..layout.power]:SetScript("OnUpdate", nil);
			_G["DHUD3_"..layout.power]:Hide();
		end
    end
end

function Pet:UPDATE_SHAPESHIFT_FORM(event, who)
	
	local maxVal, curFrac,needH, needP;
	local color = {r=0,g=0,b=0,hex=""};
	local powerType = UnitPowerType("player");
	DHUD3:Debug("Pet:UPDATE_SHAPESHIFT_FORM", powerType);
	if ( powerType ~= 0 ) then
		-- Is Animal, track mana
		_G["DHUD3_"..layout.health]:UnregisterAllEvents();
		_G["DHUD3_"..layout.health]:SetScript("OnUpdate", nil);
		_G["DHUD3_"..layout.health]:Hide();
		_G["DHUD3_"..layout.power]:UnregisterAllEvents();
		_G["DHUD3_"..layout.power]:SetScript("OnUpdate", nil);
		_G["DHUD3_"..layout.power]:Hide();
		self:UnregisterEvent("UNIT_PET");
		--Power
		bar = _G["DHUD3_"..druidBar.."po"];
	    bar.who = "player";
	    bar.bar = druidBar.."po"
		bar.display = 0;
		bar.colors = db.colors[0];
		bar.maxVal = UnitPowerMax("player", 0);
		local curFrac = UnitPower("player", 0) / bar.maxVal;
		color.r, color.g, color.b, color.hex = DHUD3:Colorize(this.colors, curFrac);
		DHUD3:UpdateBar(bar, bar.maxVal, curFrac, color);
	    maxVal, curFrac, needP = DHUD3:GetUnitPower(unit);
	    colors = db.colors[tostring(UnitPowerType(unit))];
	    color.r, color.g, color.b, color.hex = DHUD3:Colorize(colors, curFrac);
	    DHUD3:UpdateBar(powerBar, maxVal, curFrac, color);
		bar:SetScript("OnUpdate", DHUD3.OnUpdateBarPower);		
		bar:RegisterEvent("UNIT_DISPLAYPOWER");
		bar:SetScript("OnEvent", DHUD3.OnUnitPower);
		bar:SetAlpha(1);
		bar:Show();
		_G["DHUD3_"..db.druidBar.."pod_Text"]:SetAlpha(1);
    	local tagCode = (string_len(db.dCStyle) > 0) and db.dCStyle or db.dStyle;
    	DogTag:AddFontString(_G["DHUD3_"..db.druidBar.."pod_Text_Text"], _G["DHUD3_"..db.druidBar.."pod_Text"], tagCode, "Unit", { unit = "player" });
	else
		_G["DHUD3_"..druidBar.."po"]:SetScript("OnUpdate", nil);
		_G["DHUD3_"..druidBar.."po"]:SetScript("OnEvent", nil);
		_G["DHUD3_"..druidBar.."po"]:UnregisterAllEvents();
		_G["DHUD3_"..druidBar.."po"]:Hide();
		DogTag:RemoveFontString(_G["DHUD3_"..db.druidBar.."pod_Text_Text"]);
		_G["DHUD3_"..db.druidBar.."pod_Text"]:SetAlpha(0);
		if ( UnitExists("pet") ) then
			SetBars("pet", layout.health, layout.power);
		end
		self:RegisterEvent("UNIT_PET");
	end
end
